@@ -10,12 +10,15 @@
 #include <config.h>
 #endif
 
+#include <glib/gerror.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkrange.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkrange.h>
-#include <glib/gerror.h>
-
+#include <gtk/gtkmessagedialog.h> 
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-about.h>
+#include <libgnome/gnome-help.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-control.h>
@@ -23,6 +26,7 @@
 
 #include <bonobo-activation/bonobo-activation.h>
 
+#include "cddb.h"
 #include "cdrom.h"
 #include "display.h"
 #include "gnome-cd.h"
@@ -38,7 +42,7 @@ maybe_close_tray (GnomeCD *gcd)
 	GError *error;
 
 	if (gnome_cdrom_get_status (gcd->cdrom, &status, &error) == FALSE) {
-		gcd_warning ("%s", error->message);
+		gcd_warning ("%s", error);
 		g_error_free (error);
 		
 		return;
@@ -114,7 +118,7 @@ play_cb (GtkButton *button,
 			end_track = -1;
 			endmsf = NULL;
 		} else {
-			end_track == 2;
+			end_track = 2;
 			endmsf = &msf;
 		}
 		
@@ -140,20 +144,29 @@ play_cb (GtkButton *button,
 
 	switch (status->audio) {
 	case GNOME_CDROM_AUDIO_PLAY:
-		if (gnome_cdrom_pause (gcd->cdrom, &error) == FALSE) {
-			gcd_warning ("%s", error);
-			g_error_free (error);
-
-			g_free (status);
-			return;
-		}
-
-		if (gcd->current_image == gcd->pause_image) {
+		if (gcd->current_image == gcd->play_image) {
 			aob = gtk_widget_get_accessible (GTK_WIDGET (gcd->play_b));
-			atk_object_set_name (aob, _("Play"));
+			atk_object_set_name (aob, _("Pause"));
 			gtk_container_remove (GTK_CONTAINER (gcd->play_b), gcd->current_image);
-			gtk_container_add (GTK_CONTAINER (gcd->play_b), gcd->play_image);
-			gcd->current_image = gcd->play_image;
+			gtk_container_add (GTK_CONTAINER (gcd->play_b), gcd->pause_image);
+			gcd->current_image = gcd->pause_image;
+		} else { 
+			if (gnome_cdrom_pause (gcd->cdrom, &error) == FALSE) {
+				gcd_warning ("%s", error);
+				g_error_free (error);
+
+				g_free (status);
+				return;
+			}
+
+			if (gcd->current_image == gcd->pause_image) {
+				aob = gtk_widget_get_accessible (GTK_WIDGET (gcd->play_b));
+				atk_object_set_name (aob, _("Play"));
+				gtk_container_remove (GTK_CONTAINER (gcd->play_b), gcd->current_image);
+				gtk_container_add (GTK_CONTAINER (gcd->play_b),
+						   gcd->play_image);
+				gcd->current_image = gcd->play_image;
+			}
 		}
 		break;
 
@@ -415,7 +428,40 @@ rewind_release_cb (GtkButton *button,
 	
 	return FALSE;
 }
-	
+
+static void
+set_track_option_menu (GtkOptionMenu *menu,
+		       int track)
+{
+	g_signal_handlers_block_matched (G_OBJECT (menu),
+					 G_SIGNAL_MATCH_FUNC,
+					 0, 0, NULL,
+					 G_CALLBACK (skip_to_track), NULL);
+	gtk_option_menu_set_history (menu, track - 1);
+	g_signal_handlers_unblock_matched (G_OBJECT (menu),
+					   G_SIGNAL_MATCH_FUNC,
+					   0, 0, NULL,
+					   G_CALLBACK (skip_to_track), NULL);
+}
+
+static void
+set_window_track_title (GnomeCD *gcd,
+			GnomeCDRomStatus *status)
+{
+	int idx = status->track - 1;
+	const char *artist = "";
+	const char *track_name = "";
+
+	if (gcd->disc_info) {
+		if (idx >= 0 && idx < gcd->disc_info->ntracks)
+			track_name = gcd->disc_info->track_info [idx]->name;
+
+		artist = gcd->disc_info->artist ? gcd->disc_info->artist : "";
+	}
+
+	gnome_cd_set_window_title (gcd, artist, track_name);
+}
+
 /* Do all the stuff for when the status is ok */
 static void
 status_ok (GnomeCD *gcd,
@@ -441,7 +487,6 @@ status_ok (GnomeCD *gcd,
 	gtk_widget_set_sensitive (gcd->next_b, TRUE);
 	gtk_widget_set_sensitive (gcd->eject_b, TRUE);
 
-
 	switch (status->audio) {
 	case GNOME_CDROM_AUDIO_NOTHING:
 		break;
@@ -461,18 +506,8 @@ status_ok (GnomeCD *gcd,
 		/* Find out if the track has changed */
 		track = gtk_option_menu_get_history (GTK_OPTION_MENU (gcd->tracks));
 		if (track + 1 != status->track) {
-			g_signal_handlers_block_matched (G_OBJECT (gcd->tracks),
-							 G_SIGNAL_MATCH_FUNC,
-							 0, 0, NULL,
-							 G_CALLBACK (skip_to_track),
-							 gcd);
-			gtk_option_menu_set_history (GTK_OPTION_MENU (gcd->tracks),
-						     status->track - 1);
-			g_signal_handlers_unblock_matched (G_OBJECT (gcd->tracks),
-							   G_SIGNAL_MATCH_FUNC,
-							   0, 0, NULL,
-							   G_CALLBACK (skip_to_track),
-							   gcd);
+			set_track_option_menu (GTK_OPTION_MENU (gcd->tracks),
+					       status->track);
 		}
 		
 		
@@ -486,13 +521,7 @@ status_ok (GnomeCD *gcd,
 		cd_display_set_line (CD_DISPLAY (gcd->display),
 				     CD_DISPLAY_LINE_TIME, text);
 		g_free (text);
-		if (gcd->disc_info != NULL) {
-			gnome_cd_set_window_title (gcd,
-						   gcd->disc_info->artist ?
-						   gcd->disc_info->artist : "",
-						   gcd->disc_info->track_info[status->track - 1]->name ?
-						   gcd->disc_info->track_info[status->track - 1]->name : "");
-		}
+		set_window_track_title (gcd, status);
 			
 		break;
 
@@ -508,14 +537,7 @@ status_ok (GnomeCD *gcd,
 			gcd->current_image = gcd->play_image;
 		}
 
-		if (gcd->disc_info != NULL) {
-			gnome_cd_set_window_title (gcd,
-						   gcd->disc_info->artist ?
-						   gcd->disc_info->artist : "",
-						   gcd->disc_info->track_info[status->track - 1]->name ?
-						   gcd->disc_info->track_info[status->track - 1] ->name : "");
-		}
-
+		set_window_track_title (gcd, status);
 		break;
 		
 	case GNOME_CDROM_AUDIO_COMPLETE:
@@ -546,8 +568,19 @@ status_ok (GnomeCD *gcd,
 					return;
 				}
 			}
-		}
-		
+		} else {
+			/* We've stopped playing anything
+			   reset play button, and track option menu */
+			if (gcd->current_image != gcd->play_image) {
+				gtk_container_remove (GTK_CONTAINER (gcd->play_b),
+						      gcd->pause_image);
+				gtk_container_add (GTK_CONTAINER (gcd->play_b),
+						   gcd->play_image);
+				gcd->current_image = gcd->play_image;
+			}
+
+			set_track_option_menu (GTK_OPTION_MENU (gcd->tracks), 1);
+		}		
 		break;
 		
 	case GNOME_CDROM_AUDIO_STOP:
@@ -624,9 +657,6 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		      GnomeCDRomStatus *status,
 		      GnomeCD *gcd)
 {
-	char *text;
-	int track;
-
 	if (gcd->not_ready == TRUE) {
 		return;
 	}
@@ -687,6 +717,52 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
 		gnome_cd_set_window_title (gcd, NULL, NULL);
 		break;
 
+	case GNOME_CDROM_STATUS_DATA_CD:
+		if (gcd->disc_info != NULL) {
+				cddb_free_disc_info (gcd->disc_info);
+				gcd->disc_info = NULL;
+		}	
+		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
+
+                /* Set the other widgets off */
+                gtk_widget_set_sensitive (gcd->tracks, FALSE);
+                gtk_widget_set_sensitive (gcd->back_b, FALSE);
+                gtk_widget_set_sensitive (gcd->rewind_b, FALSE);
+                gtk_widget_set_sensitive (gcd->play_b, FALSE);
+                gtk_widget_set_sensitive (gcd->stop_b, FALSE);
+                gtk_widget_set_sensitive (gcd->ffwd_b, FALSE);
+                gtk_widget_set_sensitive (gcd->next_b, FALSE);
+
+		/* Eject button can be used even to eject a data cd */
+		gtk_widget_set_sensitive (gcd->eject_b, TRUE);
+	
+		cd_display_clear (CD_DISPLAY (gcd->display));
+                cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("Data CD"));
+                gnome_cd_set_window_title (gcd, NULL, NULL);
+                break;
+
+	case GNOME_CDROM_STATUS_NO_CDROM:
+		if (gcd->disc_info != NULL) {
+				cddb_free_disc_info (gcd->disc_info);
+				gcd->disc_info = NULL;
+		}	
+		gtk_widget_set_sensitive (gcd->trackeditor_b, FALSE);
+
+                /* Set the other widgets off */
+                gtk_widget_set_sensitive (gcd->tracks, FALSE);
+                gtk_widget_set_sensitive (gcd->back_b, FALSE);
+                gtk_widget_set_sensitive (gcd->rewind_b, FALSE);
+                gtk_widget_set_sensitive (gcd->play_b, FALSE);
+                gtk_widget_set_sensitive (gcd->stop_b, FALSE);
+                gtk_widget_set_sensitive (gcd->ffwd_b, FALSE);
+                gtk_widget_set_sensitive (gcd->next_b, FALSE);
+		gtk_widget_set_sensitive (gcd->eject_b, TRUE);
+	
+		cd_display_clear (CD_DISPLAY (gcd->display));
+                cd_display_set_line (CD_DISPLAY (gcd->display), CD_DISPLAY_LINE_TIME, _("No Cdrom"));
+                gnome_cd_set_window_title (gcd, NULL, NULL);
+                break;
+
 	default:
 		if (gcd->disc_info != NULL) {
 			cddb_free_disc_info (gcd->disc_info);
@@ -725,6 +801,29 @@ about_cb (GtkWidget *widget,
 	}
 }
 
+void
+help_cb (GtkWidget *widget,
+	 gpointer data)
+{
+	GError *error = NULL;
+
+	gnome_help_display ("gnome-cd", NULL, &error); 
+	if (error) {
+		GtkWidget *msgbox;
+		msgbox = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 ("There was an error displaying help: \n%s"),
+						 error->message);
+		g_signal_connect (G_OBJECT(msgbox), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_widget_show (msgbox);
+		g_error_free (error);
+	} 
+}
+ 
 void
 loopmode_changed_cb (GtkWidget *display,
 		     GnomeCDRomMode mode,
@@ -827,7 +926,7 @@ open_track_editor (GtkWidget *widget,
 	}
 
 	if (data != NULL) {
-		discid = g_strdup_printf ("%08lx", data->discid);
+		discid = g_strdup_printf ("%08lx", (gulong) data->discid);
 		GNOME_Media_CDDBTrackEditor_setDiscID (track_editor, discid, &ev);
 		g_free (discid);
 	}
@@ -851,7 +950,7 @@ volume_changed (GtkRange *range,
 
 	volume = gtk_range_get_value (range);
 
-	if (gnome_cdrom_set_volume (gcd->cdrom, (int) volume, &error) == FALSE) {
+	if (gnome_cdrom_set_volume (gcd->cdrom, (int) -volume, &error) == FALSE) {
 		gcd_warning ("Error setting volume: %s", error);
 		g_error_free (error);
 	}
