@@ -230,6 +230,7 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 {
 	gchar* temp_string1 = NULL;
 	gchar* temp_string2 = NULL;
+	mode_t old_mask;
 
 	gnome_appbar_clear_stack (GNOME_APPBAR (grecord_widgets.appbar));
 
@@ -267,8 +268,10 @@ on_stop_activate_cb (GtkWidget* widget, gpointer data)
 
 		RecEng.is_running = FALSE;
 
+		old_mask = umask(0077);
 		run_command (_("Converting file..."), sox_command, temp_string1,
 			     temp_string2, "-w", "-s", tfile1, tfile2, NULL);
+		umask(old_mask);
 
 		temp_string = g_build_filename (temp_dir, 
 						temp_filename_play, NULL);
@@ -433,7 +436,6 @@ on_exit_activate_cb (GtkWidget* widget, gpointer data)
 		choice = save_dont_or_cancel (_("Cancel"));
 		if (choice == SAVE) {
 			save_dialog ();
-			return;
 		}
 		else if (choice == CANCEL)
 			return;
@@ -646,6 +648,16 @@ record_sound (void)
 	
 	tfile = g_build_filename (temp_dir, temp_filename_record, NULL);
 	target = fopen (tfile, "w");
+	if(target == NULL) {
+		g_free (tfile);
+		return;
+	}
+
+	/* Set permissions on new file */
+	if(fchmod (fileno (target), 0600) != 0) {
+		g_free (tfile);
+		return;
+	}
 	g_free (tfile);
 
 	/* Set up bits, channels etc after the preferences */
@@ -867,6 +879,8 @@ save_filename (GtkFileSelection* selector, gpointer file_selector)
 	file_changed = FALSE;
 	save_set_sensitive (FALSE);
 	file_selector = NULL;
+
+	gtk_main_quit ();
 }
 
 /* Yes I stole this from GEdit. */
@@ -958,7 +972,7 @@ save_dont_or_cancel (const char *quit_text)
 				       GTK_MESSAGE_WARNING,
 				       GTK_BUTTONS_NONE,
 				       _("<b>Do you want to save the changes you made to \"%s\"?</b>\n"
-					 "\nYour changes will be lost is you don't save them."),
+					 "\nYour changes will be lost if you don't save them."),
 				       filename);
 	
 	grec_dialog_add_button (GTK_DIALOG (mess),
@@ -1206,6 +1220,8 @@ save_dialog (void)
 	
 	gtk_widget_show (filesel);
 
+	gtk_main ();
+
 	g_free (temp_file);
 }
 
@@ -1408,6 +1424,8 @@ check_if_sounddevice_ready ()
 					       GTK_BUTTONS_OK,
 					       _("The sound device is not ready. Please check that there "
 						 "isn't\nanother program running that is using the device."));
+		g_signal_connect (G_OBJECT (mess), "response",
+				  G_CALLBACK (gtk_widget_destroy), NULL);
 		gtk_dialog_run (GTK_DIALOG (mess));
 		return FALSE;
 	}
